==> lb_branch_hours_blocks-1.0.2.tar.gz: lb_branch_hours_blocks/src/Plugin/Block/BranchHoursBlock.php <==
<?php

namespace Drupal\lb_branch_hours_blocks\Plugin\Block;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an LB Branch Hours block.
 *
 * @Block(
 *   id = "lb_branch_hours",
 *   admin_label = @Translation("Branch Hours"),
 *   category = @Translation("Branch blocks")
 * )
 */
class BranchHoursBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Offset to show field before a holiday.
   */
  const SHOW_BEFORE_OFFSET = 1209600;

  /**
   * Offset to show field after a holiday.
   */
  const SHOW_AFTER_OFFSET = 86400;

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * System time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Location node
   *
   * @var \Drupal\node\Entity\Node
   */
  protected $node;

  /**
   * ConfigFactory
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new BranchHours instance.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The route provider.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition,
    CurrentRouteMatch $currentRouteMatch,
    ConfigFactoryInterface $configFactory,
    TimeInterface $time
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentRouteMatch = $currentRouteMatch;
    $this->configFactory = $configFactory;
    $this->time = $time;
    $node = $currentRouteMatch->getParameter('node');
    if ($node instanceof NodeInterface) {
      if ($node->bundle() === 'branch') {
        $this->node = $node;
      }
      else if ($node->hasField('field_location_reference') && !$node->get('field_location_reference')->isEmpty()) {
        $entities = $node->get('field_location_reference')->referencedEntities();
        $this->node = reset($entities);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('config.factory'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    if (!$this->node instanceof NodeInterface) {
      return [];
    }

    $branch_hours = $this->getBranchHours();
    $holiday_hours = $this->getBranchHolidayHours();

    $build = [
      'branch_name' => [
        '#type' => 'inline_template',
        '#template' => '<h1>{{ branch_name }}</h1>',
        '#context' => [
          'branch_name' => $this->node->label(),
        ],
      ],
      'branch_phone' => [
        '#type' => 'inline_template',
        '#template' => '<a href="tel:{{ branch_phone }}"><i class="fa fa-phone" aria-hidden="true"></i> {{ branch_phone }} </a>',
        '#context' => [
          'branch_phone' => $this->node->field_location_phone->value,
        ],
      ],
      'branch_address' => [
        '#type' => 'inline_template',
        '#template' => '<i class="fas fa-map-marker-alt"></i> <div>{{ branch_address }}</div>',
        '#context' => [
          'branch_address' => $this->getAddress(),
        ],
      ],
      'branch_email' => [
        '#type' => 'inline_template',
        '#template' => '<a href="mailto:{{ branch_email }}"><i class="fas fa-envelope"></i> {{ branch_email }} </a>',
        '#context' => [
          'branch_email' => $this->node->field_location_email->value,
        ],
      ],
      'branch_today_hours' => [
          '#lazy_builder' => [
            'lb_branch_hours_blocks.hours_today:generateHoursToday',
            $branch_hours['lazy_hours'],
          ],
          '#create_placeholder' => TRUE,
      ],
      'branch_hours' => [
        'title' => [
          '#type' => 'html_tag',
          '#tag' => 'h5',
          '#value' => $branch_hours['label'],
        ],
        'table' => $branch_hours['table'],
      ],
      'branch_holiday_hours' => [
        'title' => [
          '#type' => 'html_tag',
          '#tag' => 'h5',
          '#value' => $holiday_hours['label'],
        ],
        'table' => $holiday_hours['table'],
      ],
      'branch_hours_link' => $this->getBranchHoursLink(),
      '#attached' => [
        'library' => [
          'lb_branch_hours_blocks/hours_today',
          'lb_branch_hours_blocks/preferred_location',
        ],
        'drupalSettings' => [
          'lb_branch_hours_blocks' => [
            'branch_hours' => $branch_hours['js_settings'],
            'tz' => $this->configFactory->get('system.date')->get('timezone')['default'],
            'keys' => array_keys($branch_hours['js_settings']),
          ]
        ]
      ],
    ];
    return $build;
  }

  /**
   * Returns formatted Location address.
   *
   * @return \Drupal\Component\Render\MarkupInterface|string
   */
  protected function getAddress() {
    $address = $this->node->get('field_location_address')->first();
    if ($address) {
      $address_array = $address->toArray();
      $location_address = "{$address_array['address_line1']}, {$address_array['locality']}<br /> {$address_array['administrative_area']} {$address_array['postal_code']}";
      return Markup::create($location_address);
    }
    return '';
  }

  /**
   * Returns array with data for rendering Branch Hours.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getBranchHours() {
    $lazy_hours = [];
    $js_settings = [];
    $rows = $table = [];
    $label = '';

    if ($this->node->hasField('field_branch_hours')) {
      $branch_hours = $this->node->field_branch_hours->get(0)->getValue();
      unset($branch_hours['_attributes']);
      $groups = [];
      foreach ($branch_hours as $key => $i_item) {
        // Do not process label. Store it name for later usage.
        if ($key === 'hours_label') {
          $label = $i_item;
          $js_key = str_ireplace('-', '_', Html::getClass($i_item));
          continue;
        }

        $day = str_replace('hours_', '', $key);
        $value = $i_item ?? $this->t('Closed');
        $lazy_hours[$day] = $value;
        if ($groups && end($groups)['value'] == $value) {
          $array_keys = array_keys($groups);
          $group = &$groups[end($array_keys)];
          $group['days'][] = $day;
        }
        else {
          $groups[] = [
            'value' => $value,
            'days' => [$day],
          ];
        }

        // Populate JS hours.
        if (stripos($key, 'hours_') === 0) {
          $js_settings[$js_key][] = $value;
        }
      }

      foreach ($groups as $group_item) {
        $title = sprintf('%s - %s', ucfirst(reset($group_item['days'])), ucfirst(end($group_item['days'])));
        if (count($group_item['days']) == 1) {
          $title = ucfirst(reset($group_item['days']));
        }
        $hours = $group_item['value'];
        $rows[] = [$title, $hours];
      }

      if (!empty($rows)) {
        $table = [
          '#theme' => 'table',
          '#header' => [],
          '#rows' => $rows,
        ];
      }
    }

    return [
      'lazy_hours' => $lazy_hours,
      'label' => $label,
      'table' => $table,
      'js_settings' => $js_settings,
    ];
  }

  /**
   * Returns array with data for rendering Holidays Hours.
   *
   * @return array
   * @throws \Exception
   */
  protected function getBranchHolidayHours() {
    $rows = [];
    $config = $this->configFactory->get('openy_field_holiday_hours.settings');

    // Calculate timezone offset.
    $dt = new DrupalDateTime('now', $this->configFactory->get('system.date')->get('timezone')['default']);
    $tz_offset = $dt->getOffset();

    // The Holiday Hours should be shown before N days.
    $show_before_offset = $config->get('show_before_offset') ?: self::SHOW_BEFORE_OFFSET;
    $show_before_offset = $tz_offset + $show_before_offset;

    // Also the Holiday Hours should be shown during N offset after a holiday.
    $show_after_offset = $config->get('show_after_offset') ?: self::SHOW_AFTER_OFFSET;
    $request_time = $this->time->getRequestTime();
    if ($this->node->hasField('field_branch_holiday_hours')) {
      $holiday_hours = $this->node->field_branch_holiday_hours->getValue();
      foreach ($holiday_hours as $holiday) {
        // Skip holidays with empty date.
        if (empty($holiday['date'])) {
          continue;
        }
        $holiday_timestamp = $holiday['date'];
        if ($request_time < ($holiday_timestamp + $show_after_offset) && ($holiday_timestamp - $request_time) <= $show_before_offset) {
          $title = Html::escape($holiday['holiday']);
          $rows[] = [
            'data' => [
              $title,
              $holiday['hours'],
            ],
            'data-timestamp' => $holiday_timestamp,
          ];
        }
      }
      $table = [
        '#theme' => 'table',
        '#rows' => $rows,
        '#cache' => [
          'tags' => ['ymca_cron'],
        ],
      ];

    }
    return [
      'label' => $this->t('Holiday Hours'),
      'table' => $table ?? []
    ];
  }

  /**
   * Returns More Hours link render array.
   *
   * @return array
   */
  protected function getBranchHoursLink() {
    if ($this->node->hasField('field_more_hours_link')) {
      return $this->node->field_more_hours_link->view(['label' => 'hidden']);
    }
    return [];
  }

}

==> lb_branch_hours_blocks-1.0.6.tar.gz: lb_branch_hours_blocks/src/Plugin/Block/BranchHoursBlock.php <==
<?php

namespace Drupal\lb_branch_hours_blocks\Plugin\Block;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Http\RequestStack;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an LB Branch Hours block.
 *
 * @Block(
 *   id = "lb_branch_hours",
 *   admin_label = @Translation("Branch Hours"),
 *   category = @Translation("Branch blocks")
 * )
 */
class BranchHoursBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Offset to show field before a holiday.
   */
  const SHOW_BEFORE_OFFSET = 1209600;

  /**
   * Offset to show field after a holiday.
   */
  const SHOW_AFTER_OFFSET = 86400;

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * System time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Location node
   *
   * @var \Drupal\node\Entity\Node
   */
  protected $node;

  /**
   * ConfigFactory
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new BranchHours instance.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The route provider.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition,
    CurrentRouteMatch $currentRouteMatch,
    ConfigFactoryInterface $configFactory,
    TimeInterface $time,
    RequestStack $requestStack,
    EntityTypeManager $entityTypeManager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentRouteMatch = $currentRouteMatch;
    $this->configFactory = $configFactory;
    $this->time = $time;
    $node = $currentRouteMatch->getParameter('node');
    if(!$node && ($destination = $requestStack->getCurrentRequest()->query->get('destination'))) {
      $params = Url::fromUserInput($destination)->getRouteParameters();
      if (isset($params['node'])) {
        $node = $entityTypeManager->getStorage('node')->load($params['node']);
      }
    }

    if ($node instanceof NodeInterface) {
      if ($node->bundle() === 'branch') {
        $this->node = $node;
      }
      else if ($node->hasField('field_location_reference') && !$node->get('field_location_reference')->isEmpty()) {
        $entities = $node->get('field_location_reference')->referencedEntities();
        $this->node = reset($entities);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('config.factory'),
      $container->get('datetime.time'),
      $container->get('request_stack'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    if (!$this->node instanceof NodeInterface) {
      return [];
    }

    $branch_hours = $this->getBranchHours();
    $holiday_hours = $this->getBranchHolidayHours();

    $build = [
      'branch_name' => [
        '#type' => 'inline_template',
        '#template' => '<h1>{{ branch_name }}</h1>',
        '#context' => [
          'branch_name' => $this->node->label(),
        ],
      ],
      'branch_phone' => [
        '#type' => 'inline_template',
        '#template' => '<a href="tel:{{ branch_phone }}"><i class="fa fa-phone" aria-hidden="true"></i> {{ branch_phone }} </a>',
        '#context' => [
          'branch_phone' => $this->node->field_location_phone->value,
        ],
      ],
      'branch_address' => [
        '#type' => 'inline_template',
        '#template' => '<i class="fas fa-map-marker-alt"></i> <div>{{ branch_address }}</div>',
        '#context' => [
          'branch_address' => $this->getAddress(),
        ],
      ],
      'branch_email' => [
        '#type' => 'inline_template',
        '#template' => '<a href="mailto:{{ branch_email }}"><i class="fas fa-envelope"></i> {{ branch_email }} </a>',
        '#context' => [
          'branch_email' => $this->node->field_location_email->value,
        ],
      ],
      'branch_today_hours' => [
          '#lazy_builder' => [
            'lb_branch_hours_blocks.hours_today:generateHoursToday',
            $branch_hours['lazy_hours'],
          ],
          '#create_placeholder' => TRUE,
      ],
      'branch_hours' => [
        'title' => [
          '#type' => 'html_tag',
          '#tag' => 'h5',
          '#value' => $branch_hours['label'],
        ],
        'table' => $branch_hours['table'],
      ],
      'branch_holiday_hours' => [
        'title' => [
          '#type' => 'html_tag',
          '#tag' => 'h5',
          '#value' => $holiday_hours['label'],
        ],
        'table' => $holiday_hours['table'],
      ],
      'branch_hours_link' => $this->getBranchHoursLink(),
      '#attached' => [
        'library' => [
          'lb_branch_hours_blocks/hours_today',
          'lb_branch_hours_blocks/preferred_location',
        ],
        'drupalSettings' => [
          'lb_branch_hours_blocks' => [
            'branch_hours' => $branch_hours['js_settings'],
            'tz' => $this->configFactory->get('system.date')->get('timezone')['default'],
            'keys' => array_keys($branch_hours['js_settings']),
          ]
        ]
      ],
    ];
    return $build;
  }

  /**
   * Returns formatted Location address.
   *
   * @return \Drupal\Component\Render\MarkupInterface|string
   */
  protected function getAddress() {
    $address = $this->node->get('field_location_address')->first();
    if ($address) {
      $address_array = $address->toArray();
      $location_address = "{$address_array['address_line1']}, {$address_array['locality']}<br /> {$address_array['administrative_area']} {$address_array['postal_code']}";
      return Markup::create($location_address);
    }
    return '';
  }

  /**
   * Returns array with data for rendering Branch Hours.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getBranchHours() {
    $lazy_hours = [];
    $js_settings = [];
    $rows = $table = [];
    $label = '';

    if ($this->node->hasField('field_branch_hours')) {
      $branch_hours = $this->node->field_branch_hours->get(0)->getValue();
      unset($branch_hours['_attributes']);
      $groups = [];
      foreach ($branch_hours as $key => $i_item) {
        // Do not process label. Store it name for later usage.
        if ($key === 'hours_label') {
          $label = $i_item;
          $js_key = str_ireplace('-', '_', Html::getClass($i_item));
          continue;
        }

        $day = str_replace('hours_', '', $key);
        $value = $i_item ?? $this->t('Closed');
        $lazy_hours[$day] = $value;
        if ($groups && end($groups)['value'] == $value) {
          $array_keys = array_keys($groups);
          $group = &$groups[end($array_keys)];
          $group['days'][] = $day;
        }
        else {
          $groups[] = [
            'value' => $value,
            'days' => [$day],
          ];
        }

        // Populate JS hours.
        if (stripos($key, 'hours_') === 0) {
          $js_settings[$js_key][] = $value;
        }
      }

      foreach ($groups as $group_item) {
        $title = sprintf('%s - %s', ucfirst(reset($group_item['days'])), ucfirst(end($group_item['days'])));
        if (count($group_item['days']) == 1) {
          $title = ucfirst(reset($group_item['days']));
        }
        $hours = $group_item['value'];
        $rows[] = [$title, $hours];
      }

      if (!empty($rows)) {
        $table = [
          '#theme' => 'table',
          '#header' => [],
          '#rows' => $rows,
        ];
      }
    }

    return [
      'lazy_hours' => $lazy_hours,
      'label' => $label,
      'table' => $table,
      'js_settings' => $js_settings,
    ];
  }

  /**
   * Returns array with data for rendering Holidays Hours.
   *
   * @return array
   * @throws \Exception
   */
  protected function getBranchHolidayHours() {
    $rows = [];
    $config = $this->configFactory->get('openy_field_holiday_hours.settings');

    // Calculate timezone offset.
    $dt = new DrupalDateTime('now', $this->configFactory->get('system.date')->get('timezone')['default']);
    $tz_offset = $dt->getOffset();

    // The Holiday Hours should be shown before N days.
    $show_before_offset = $config->get('show_before_offset') ?: self::SHOW_BEFORE_OFFSET;
    $show_before_offset = $tz_offset + $show_before_offset;

    // Also the Holiday Hours should be shown during N offset after a holiday.
    $show_after_offset = $config->get('show_after_offset') ?: self::SHOW_AFTER_OFFSET;
    $request_time = $this->time->getRequestTime();
    if ($this->node->hasField('field_branch_holiday_hours')) {
      $holiday_hours = $this->node->field_branch_holiday_hours->getValue();
      foreach ($holiday_hours as $holiday) {
        // Skip holidays with empty date.
        if (empty($holiday['date'])) {
          continue;
        }
        $holiday_timestamp = $holiday['date'];
        if ($request_time < ($holiday_timestamp + $show_after_offset) && ($holiday_timestamp - $request_time) <= $show_before_offset) {
          $title = Html::escape($holiday['holiday']);
          $rows[] = [
            'data' => [
              $title,
              $holiday['hours'],
            ],
            'data-timestamp' => $holiday_timestamp,
          ];
        }
      }
      $table = [
        '#theme' => 'table',
        '#rows' => $rows,
        '#cache' => [
          'tags' => ['ymca_cron'],
        ],
      ];

    }
    return [
      'label' => $this->t('Holiday Hours'),
      'table' => $table ?? []
    ];
  }

  /**
   * Returns More Hours link render array.
   *
   * @return array
   */
  protected function getBranchHoursLink() {
    if ($this->node->hasField('field_more_hours_link')) {
      return $this->node->field_more_hours_link->view(['label' => 'hidden']);
    }
    return [];
  }

}
